# -*- coding: utf-8 -*-
#
# Copyright (c) 2021, HILSTER GmbH, Germany - https://hilster.io
# All rights reserved.
#
from htf.daqmx.core import DAQError
from htf.daqmx.task import Task
from ctypes import *
from htf.daqmx.api import float64, DAQmx_Val_Volts, DAQmx_Val_Cfg_Default, DAQmx_Val_Diff
from htf.mixins.delegation import DelegateMixin


class AnalogVoltageInputSingleLine(DelegateMixin):
    """
    ``AnalogVoltageInputSingleLine`` is used to access a single analog output line
    on a DAQmx device.

    A DAQmx task is automatically created and deleted in the constructor and destructor.
    """

    def __init__(self,
                 descriptor,
                 minimum_voltage=-10.0,
                 maximum_voltage=10.0,
                 terminal_config=DAQmx_Val_Cfg_Default):
        """
        Args:
            descriptor (str): The DAQmx device and line descriptor, eg.
                `cDaq1Mod1/line0`
            minimum_voltage=-10.0 (float): the minimum voltage.
            maximum_voltage=10.0 (float): the maximum voltage.
            terminal_config=DAQmx_Val_Cfg_Default: the terminal configuration
                for DAQmxCreateAIVoltageChan(). Possible values: DAQmx_Val_RSE,
                DAQmx_Val_NRSE, DAQmx_Val_Diff, DAQmx_Val_PseudoDiff.

        """
        task_name = "AI_" + descriptor.replace("/", "_")
        self.tmpvalue = float64(0.0)
        self.value = 0.0
        self.task = Task(task_name)

        if not isinstance(descriptor, bytes):
            descriptor = descriptor.encode()

        self.task.CreateAIVoltageChan(descriptor,
                                      None,
                                      terminal_config,
                                      float64(minimum_voltage),
                                      float64(maximum_voltage),
                                      DAQmx_Val_Volts,
                                      None)

    def get(self):
        """
        Return the current pin value.
        """
        self.task.ReadAnalogScalarF64(float64(1.0), byref(self.tmpvalue), None)
        self.value = self.tmpvalue.value
        return self.value

    def set(self, state=True):
        raise NotImplementedError(
            "There is no meaningful implementation for "
            "a 'set' method in the context of this class.")

    def close(self):
        """
        Close the abstractor by clearing the task.
        """
        try:
            self.task.ClearTask()
        except DAQError:
            pass

    def __del__(self):
        self.close()

    def __str__(self):
        return str(self.get())


if __name__ == "__main__":
    ai0 = AnalogVoltageInputSingleLine("Dev1/ai0", minimum_voltage=0.0,
                                       maximum_voltage=10.0, terminal_config=DAQmx_Val_Diff)

    for ii in range(10000):
        print("ai0:", ai0.get())
